using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;
using System.IO;
using System.Drawing.Printing;


namespace PrintingDemoCS
{
	/// <summary>
	/// Summary description for TextFilePrintDocument.
	/// </summary>
	public class TextFilePrintDocument : System.Drawing.Printing.PrintDocument
	{
		/// <summary> 
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;
		private Font printFont = null;
		private StreamReader streamToPrint = null;

		public TextFilePrintDocument()
		{
			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();

			// TODO: Add any initialization after the InitForm call

		}

		public TextFilePrintDocument(StreamReader streamToPrint) : base ()  
		{
			this.streamToPrint = streamToPrint ;
		}

		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Component Designer generated code
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			// 
			// TextFilePrintDocument
			// 
		}
		#endregion


		//Override OnBeginPrint to set up the font we are going to use
		protected override void OnBeginPrint(PrintEventArgs ev) 
		{
			base.OnBeginPrint(ev) ;
			printFont = new Font("Comic Sans MS", 10);
		}


		//Override the OnPrintPage to provide the printing logic for the document
		protected override void OnPrintPage(PrintPageEventArgs ev) 
		{
			base.OnPrintPage(ev) ;

			float lpp = 0 ;
			float yPos =  0 ;
			int count = 0 ;
			float leftMargin = ev.MarginBounds.Left;
			float topMargin = ev.MarginBounds.Top;
			string line = null;

			//Work out the number of lines per page
			//Use the MarginBounds on the event to do this
			lpp = ev.MarginBounds.Height  / printFont.GetHeight(ev.Graphics) ;

			//Now iterate over the file printing out each line
			//NOTE WELL: This assumes that a single line is not wider than the page width
			//Check count first so that we don't read line that we won't print
			while (count < lpp && ((line=streamToPrint.ReadLine()) != null)) 
			{
				yPos = topMargin + (count * printFont.GetHeight(ev.Graphics));
				ev.Graphics.DrawString (line, printFont, Brushes.Black, leftMargin, yPos, new StringFormat());
				count++;
			}

			//If we have more lines then print another page
			ev.HasMorePages  = (line != null);				
		}
	}
}
