using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;

namespace SortableListView
{
	/// <summary>
	/// Summary description for SortableListView.
	/// </summary>
	public class SortableListView : System.Windows.Forms.ListView
	{
		/// <summary> 
		/// Required designer variable.
		/// </summary>		
		private System.ComponentModel.Container components = null;

		private int sortColumn = -1;

		public SortableListView()
		{
			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();

			// TODO: Add any initialization after the InitForm call
			this.ColumnClick += new System.Windows.Forms.ColumnClickEventHandler(this.Base_ColumnClick);
			this.View = System.Windows.Forms.View.Details;
		}

		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Component Designer generated code
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			components = new System.ComponentModel.Container();
		}
		#endregion

		public void AddItems(params string [] stringArray)
		{
			this.Items.Add(new ListViewItem(stringArray));
		}

		#region ListViewItemComparer
		class ListViewItemComparer : IComparer 
		{
			private int col;
			private SortOrder order;

			public ListViewItemComparer() 
			{
				col = 0;
				order = SortOrder.Ascending;
			}

			public ListViewItemComparer(int column, SortOrder order) 
			{
				col = column;
				this.order = order;
			}
			
			public int Compare(object x, object y) 
			{
				int returnVal = 0;
				string firstItem = ((ListViewItem)x).SubItems[col].Text;
				string secondItem = ((ListViewItem)y).SubItems[col].Text;
				
				// Determine whether the type being compared is a date type.
				try
				{
					// Parse the two objects passed as a parameter as a DateTime.
					System.DateTime firstDate = DateTime.Parse(firstItem);
					System.DateTime secondDate = DateTime.Parse(secondItem);
					// Compare the two dates.
					returnVal = DateTime.Compare(firstDate, secondDate);
				}
					// If neither compared object has a valid date format, compare
					// as an integer or string.
				catch 
				{
					try
					{						
						int firstInt = int.Parse(firstItem);
						int secondInt = int.Parse(secondItem);
						if (firstInt < secondInt)
							returnVal = -1;
						if (firstInt == secondInt)
							returnVal = 0;
						if (firstInt > secondInt)
							returnVal = 1;
					}
					catch
					{
						// Compare the two items as a string.
						returnVal = string.Compare(firstItem, secondItem);					
					}
				}

				// Determine whether the sort order is descending.
				if (order == SortOrder.Descending)
					// Invert the value returned by String.Compare.
					returnVal *= -1;

				return returnVal;
			}
		}
		#endregion

		#region Base_ColumnClick
		private void Base_ColumnClick(object sender, System.Windows.Forms.ColumnClickEventArgs e)
		{
			// Determine whether the column is the same as the last column clicked.
			if (e.Column != sortColumn)
			{
				// Set the sort column to the new column.
				sortColumn = e.Column;
				// Set the sort order to ascending by default.
				this.Sorting = SortOrder.Ascending;
			}
			else
			{
				// Determine what the last sort order was and change it.
				if (this.Sorting == SortOrder.Ascending)
					this.Sorting = SortOrder.Descending;
				else
					this.Sorting = SortOrder.Ascending;
			}
			
			// Set the ListViewItemSorter property to a new ListViewItemComparer
			// object.
			this.ListViewItemSorter = new ListViewItemComparer(e.Column, this.Sorting);

			// Call the sort method to manually sort.
			this.Sort();
		}
		#endregion
	}
}
